__global__ 
void applyScaleFactors(
    unsigned char * const im, 
	const double * factor,
    unsigned int const numRows,
    unsigned int const numCols )
{
    // Work out which column of the image we are working on.
    const int colIdx = blockIdx.y;
    const int sliceIdx = blockIdx.z;

    // Compute the index of the start of this column
    const unsigned int startIdx = colIdx*numRows + sliceIdx*numRows*numCols;

    // We are looking at one column of one color, so the scale factor is
    // constant
    const double scale = factor[sliceIdx];

    // Loop through processing chunks of this column
    for (int rowIdx = threadIdx.x; rowIdx < numRows; rowIdx += blockDim.x)
    {
        // Compute the index of my element
        const unsigned int linearIdx = startIdx + rowIdx;

        // Scale the value with the appropriate factor
        double result = double(im[linearIdx])*scale;

        // Cap our values to the interval [0,255]
        result = result > 255.0 ? 255.0 : result;

        // Write the value to global memory.
        im[linearIdx] =  (unsigned char)rint(result);
    }
}
